-- ----------------------
-- KAPITEL 11 (Oracle 21)
-- ----------------------

-- 11.4 Oracle Spatial GeoRaster

-- Initialisierung: 
EXECUTE SDO_GEOR_ADMIN.enableGeoRaster();

-- Überprüfung: 
SELECT SDO_GEOR_ADMIN.isGeoRasterEnabled FROM DUAL;

-- 11.4.1.2 Anlegen von Tabellen

-- Tabelle mit GeoRaster-Attribut anlegen:
CREATE TABLE GeoRasterkarten (
  id         DECIMAL(8),                  -- ID
  quelle     VARCHAR(64)  NOT NULL,       -- Quelle der Karte
  georaster  SDO_GEORASTER,               -- das GeoRaster
  CONSTRAINT pk_georst PRIMARY KEY (id)   -- Primärschlüssel
);

-- Raster-Objekttabelle anlegen:
CREATE TABLE Raster OF SDO_RASTER (
  CONSTRAINT pk_rst PRIMARY KEY (rasterId, pyramidLevel, bandBlockNumber,
                                 rowBlockNumber, columnBlockNumber)
)
LOB(rasterblock) STORE AS (
  NOCACHE NOLOGGING
);

-- 11.4.1.3 Laden von Rasterdaten

-- Hier (spätestens) Karte wms_dtk500.png über Java als BLOB speichern.

DECLARE
  geor  SDO_GEORASTER;  -- das GeoRaster
  name  VARCHAR(80);    -- Name der Rasterkarte
  karte BLOB;           -- deren Daten als BLOB
  wf    CLOB;           -- deren Georeferenzierung als World File
BEGIN
  SELECT name, karte, worldfile INTO name, karte, wf 
         FROM Rasterkarten WHERE id = 1;
  INSERT INTO GeoRasterkarten VALUES (1, name, SDO_GEOR.init('RASTER', 1) );
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 1;
  SDO_GEOR.importFrom(geor, 'blocking=false, srid=31467', 'PNG', karte,
                      'WORLDFILE', wf);
  UPDATE GeoRasterkarten SET georaster = geor WHERE id = 1;
  COMMIT;
END;
/

-- vom DB-Administrator müssen Rechte vergeben werden 
-- Verzeichnis nachfolgend ggf. anpassen
 
CALL dbms_java.grant_permission( <Name des Benutzerkontos>,
     'SYS:java.io.FilePermission', 'C:\temp\*', 'read' );
CALL dbms_java.grant_permission( 'MDSYS',
     'SYS:java.io.FilePermission', 'C:\temp\*', 'read' );

-- Rasterdatei vg250_gem.png laden: 
DECLARE
  geor SDO_GEORASTER;
BEGIN
  INSERT INTO GeoRasterkarten VALUES (2, 'vg250_gem.png',
                                      SDO_GEOR.init('RASTER', 2) );
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 2;
  SDO_GEOR.importFrom(geor, 'blocking=false', 'PNG', 'FILE',
    'C:\temp\vg250_gem.png', 'WORLDFILE', 'FILE',
    'C:\temp\vg250_gem.pgw,31467');
  UPDATE GeoRasterkarten SET georaster = geor WHERE id = 2;
  COMMIT;
END;
/

-- Rasterdatei dtk500.gif laden: 
DECLARE
  geor SDO_GEORASTER;
BEGIN
  INSERT INTO GeoRasterkarten VALUES (3, 'dtk500.gif',
                                      SDO_GEOR.init('RASTER', 3) );
  SELECT georaster INTO geor FROM GeoRasterkarten WHERE id = 3;
  SDO_GEOR.importFrom(geor, 'blocking=false', 'GIF', 'FILE',
    'C:\temp\dtk500.gif', 'WORLDFILE', 'FILE', 'C:\temp\dtk500.gfw,31467');
  UPDATE GeoRasterkarten SET georaster = geor WHERE id = 3;
  COMMIT;
END;
/

INSERT INTO GeoRasterkarten
            VALUES (4, 'sentinel2.tif', SDO_GEOR.init('RASTER',4));
COMMIT;

-- Hier über Spatial Database Viewer TIFF-Datei sentinel2.tif einlesen.

-- 11.4.1.4 Daten in SDO_GEORASTER und SDO_RASTER

SELECT id, quelle, (georaster).rasterType AS type,
       (georaster).rasterDataTable AS tab,
       (georaster).rasterID AS rid, (georaster).spatialExtent AS extent
FROM GeoRasterkarten ORDER BY id;

UPDATE GeoRasterkarten r
SET r.georaster.spatialExtent = SDO_GEOR.generateSpatialExtent(georaster)
WHERE r.georaster.spatialExtent IS NULL;

COMMIT;

SELECT id, quelle, (georaster).metadata FROM GeoRasterkarten;

SELECT id, quelle, rasterID, count(*)
FROM GeoRasterkarten INNER JOIN Raster
ON (georaster).rasterID = rasterID
GROUP BY id, quelle, rasterID; 

-- 11.4.2.1 Räumliche Indexierung und Anfragen

-- Metadaten einfügen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GeoRasterkarten','georaster.spatialextent', SDO_DIM_ARRAY(
    SDO_DIM_ELEMENT('Rechtswert',3200000,4000000,1),
    SDO_DIM_ELEMENT('Hochwert',5200000,6500000,1) ) , 31467);
COMMIT;

-- Index anlegen:
CREATE INDEX GeoRasterkarten_georst_ix
ON GeoRasterkarten(georaster.spatialextent)
INDEXTYPE IS MDSYS.SPATIAL_INDEX_V2;

-- Räumliche Anfrage:
SELECT quelle FROM GeoRasterkarten gr
WHERE SDO_FILTER (gr.georaster.spatialextent, 
   SDO_GEOMETRY(2003,31467,NULL, SDO_ELEM_INFO_ARRAY(1,1003,3),
           SDO_ORDINATE_ARRAY(337000,5700000,3600000,6000000))) = 'TRUE';

-- 11.4.2.2 Kachelung

DECLARE
  q SDO_GEORASTER;     -- das Quell-GeoRaster
  z SDO_GEORASTER;     -- das Ziel-GeoRaster
BEGIN
  INSERT INTO GeoRasterkarten
    VALUES (5,'wms_dtk500.png 75x75',SDO_GEOR.init('Raster',5));
  SELECT georaster INTO q FROM GeoRasterkarten
                   WHERE quelle='wms_dtk500.png';
  SELECT georaster INTO z FROM GeoRasterkarten WHERE id = 5;
  SDO_GEOR.changeFormatCopy(q,'blockSize=(75,75,3)',z);
  UPDATE GeoRasterkarten SET georaster = z WHERE id = 5;
  COMMIT;
END;
/

-- Kachelung überprüfen (SDO_GEOM_MIN/MAX_X/Y ab Oracle 21c): 
SELECT bandBlockNumber AS B, rowBlockNumber AS r, columnBlockNumber AS C,
       SDO_GEOM_MIN_X(blockMBR) AS X1, SDO_GEOM_MIN_Y(blockMBR) AS Y1,
       SDO_GEOM_MAX_X(blockMBR) AS X2, SDO_GEOM_MAX_Y(blockMBR) AS Y2 
FROM Raster WHERE rasterID = 5
ORDER BY rowBlockNumber, columnBlockNumber, bandBlockNumber;

-- 11.4.2.3 Abfrage von Rasterzellen

-- Farbwerte über Pixelposition bestimmen:
SELECT SDO_GEOR.getCellValue(georaster, 0,450,299,0) AS R,
       SDO_GEOR.getCellValue(georaster, 0,450,299,1) AS G,
       SDO_GEOR.getCellValue(georaster, 0,450,299,2) AS B
FROM GeoRasterkarten gr WHERE quelle = 'wms_dtk500.png';

-- Pixelposition für Punkt bestimmen:
SELECT SDO_GEOR.getCellCoordinate(georaster, 0,
                          SDO_GEOMETRY('POINT(3450000 5880000)', 31467))
FROM GeoRasterkarten WHERE quelle = 'wms_dtk500.png';

-- Funktion zum Bestimmen eines Farbwertes aus einer Farbtabelle:
CREATE OR REPLACE FUNCTION colorValue (i DECIMAL, map SDO_GEOR_COLORMAP)
RETURN VARCHAR IS
BEGIN
  RETURN map.red(i) || ' ' || map.green(i) || ' ' || map.blue(i);
END;
/

-- Nutzung der Funktion:
SELECT colorValue(SDO_GEOR.getCellValue(georaster,0,450,299,0),
                  SDO_GEOR.getColorMap(georaster,1))
FROM GeoRasterkarten WHERE quelle = 'dtk500.gif';

-- 11.4.2.4 Ausschnittbildung

DECLARE
  q SDO_GEORASTER;    -- das Quell-GeoRaster
  z SDO_GEORASTER;    -- das Ziel-GeoRaster
BEGIN
  INSERT INTO GeoRasterkarten
    VALUES (6,'wms_dtk500.png subset',SDO_GEOR.init('Raster',6));
  SELECT georaster INTO q FROM GeoRasterkarten
                   WHERE quelle = 'wms_dtk500.png 75x75';
  SELECT georaster INTO z FROM GeoRasterkarten WHERE id = 6;
  SDO_GEOR.subset(q, SDO_GEOMETRY(2003,31467,null,
                    SDO_ELEM_INFO_ARRAY(1,1003,3),
                    SDO_ORDINATE_ARRAY(344000,5880000,3460000,5900000)),
                    '1-4','blocking=false',z);
  UPDATE GeoRasterkarten SET georaster = z WHERE id = 6;
END;
/

-- 11.4.2.5 Bildpyramide

DECLARE
  q   SDO_GEORASTER;    -- das Quell-GeoRaster
  z   SDO_GEORASTER;    -- das Ziel-GeoRaster
BEGIN
  INSERT INTO GeoRasterkarten
    VALUES (7,'sentinel2.tif pyramid',SDO_GEOR.init('Raster',7));
  SELECT georaster INTO q FROM GeoRasterkarten
                   WHERE quelle = 'sentinel2.tif';
  SELECT georaster INTO z FROM GeoRasterkarten WHERE id = 7;
  SDO_GEOR.changeFormatCopy (q, 'blocking=false', z);
  SDO_GEOR.generatePyramid  (z, 'rLevel=3, resampling=CUBIC');
  UPDATE GeoRasterkarten SET georaster = z WHERE id = 7;
  COMMIT;
END;
/

-- Bildpyramide überprüfen (SDO_GEOM_MIN/MAX_X/Y ab Oracle 21c): 
SELECT pyramidLevel AS l, rowBlockNumber AS r, columnBlockNumber AS C,
       SDO_GEOM_MIN_X(blockMBR) AS X1, SDO_GEOM_MIN_Y(blockMBR) AS Y1,
       SDO_GEOM_MAX_X(blockMBR) AS X2, SDO_GEOM_MAX_Y(blockMBR) AS Y2 
FROM Raster WHERE rasterID = 7
ORDER BY pyramidLevel;

-- 11.4.2.6 Kompression

DECLARE
  q   SDO_GEORASTER;   -- das Quell-GeoRaster
  z   SDO_GEORASTER;   -- das Ziel-GeoRaster
BEGIN
  DELETE FROM GeoRasterkarten WHERE id = 8;
  INSERT INTO GeoRasterkarten
    VALUES (8,'wms_dtk500.png compr',SDO_GEOR.init('Raster',8));
  SELECT georaster INTO q FROM GeoRasterkarten 
                          WHERE quelle = 'wms_dtk500.png';
  SELECT georaster INTO z FROM GeoRasterkarten WHERE id = 8;
  SDO_GEOR.changeFormatCopy(q, 'compression=JPEG-F quality=10', z);
  UPDATE GeoRasterkarten SET georaster = z WHERE id = 8;
  COMMIT;
END;
/

DECLARE
  q   SDO_GEORASTER;   -- das Quell-GeoRaster
  z   SDO_GEORASTER;   -- das Ziel-GeoRaster
BEGIN
  DELETE FROM GeoRasterkarten WHERE id = 8;
  INSERT INTO GeoRasterkarten
    VALUES (8,'wms_dtk500.png compr',SDO_GEOR.init('Raster',8));
  SELECT georaster INTO q FROM GeoRasterkarten 
                          WHERE quelle = 'wms_dtk500.png';
  SELECT georaster INTO z FROM GeoRasterkarten WHERE id = 8;
  SDO_GEOR.compressJP2(q,'rlevel=4 ratio=(60,40,20,10) tileSize=(150,150)',z);
  UPDATE GeoRasterkarten SET georaster = z WHERE id = 8;
  COMMIT;
END;
/

-- 11.4.2.7 Export von Rasterdaten

-- vom DB-Administrator müssen Rechte vergeben werden: 
-- Verzeichnis nachfolgend ggf. anpassen

CALL dbms_java.grant_permission( <Name des Benutzerkontos>,
     'SYS:java.io.FilePermission', 'C:\temp\*', 'write' );
CALL dbms_java.grant_permission( 'MDSYS', 
     'SYS:java.io.FilePermission', 'C:\temp\*', 'write' );
	 
-- Export durchführen: 
DECLARE
  geor SDO_GEORASTER;
BEGIN
  SELECT georaster INTO geor FROM GeoRasterkarten 
                             WHERE quelle = 'sentinel2.tif pyramid';
  SDO_GEOR.exportTo(geor, 'pLevel=1','PNG', 'FILE',
                    'C:\temp\sentinel2_level1');
END;
/